"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const go_1 = require("@kunn/go");
const server_1 = require("@kunn/server");
const typescript_1 = require("@kunn/typescript");
const coco_1 = require("@sudoo/coco");
const io_1 = require("@sudoo/io");
const http_1 = require("http");
const read_1 = require("./config/read");
const util_1 = require("./generator/util");
exports.KunnCLI = (args) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const coco = coco_1.Coco.create();
        coco.command(coco_1.createInfoCommand('help', coco, console.log));
        coco.command(coco_1.Command
            .create('typescript')
            .argument(coco_1.Argument.create('config'))
            .option(coco_1.Option.create('o').setName('out').required())
            .then((inputs) => __awaiter(void 0, void 0, void 0, function* () {
            const kunn = yield read_1.fromConfig(inputs.config);
            const definition = kunn.routes().map((route) => typescript_1.generateTypeScriptGesture(route)).join('\n');
            yield io_1.writeTextFile(util_1.parseTargetPath(inputs.out, 'd.ts'), definition);
        })));
        coco.command(coco_1.Command
            .create('go')
            .argument(coco_1.Argument.create('config'))
            .option(coco_1.Option.create('o').setName('out').required())
            .then((inputs) => __awaiter(void 0, void 0, void 0, function* () {
            const kunn = yield read_1.fromConfig(inputs.config);
            const definition = kunn.routes().map((route) => go_1.generateGoLangGesture(route)).join('\n');
            yield io_1.writeTextFile(util_1.parseTargetPath(inputs.out, 'go'), definition);
        })));
        coco.command(coco_1.Command
            .create('serve')
            .argument(coco_1.Argument.create('config'))
            .then((inputs) => __awaiter(void 0, void 0, void 0, function* () {
            const kunn = yield read_1.fromConfig(inputs.config);
            const server = server_1.createKunnServer(kunn);
            http_1.createServer(server).listen(8000);
        })));
        yield coco.go(args);
    }
    catch (error) {
        if (process.env.NODE_ENV === 'development') {
            console.log(error);
        }
        else {
            console.log(error.message);
        }
    }
});
exports.default = exports.KunnCLI;
